/*
  File name        : tools.h
  Date of creation : 18/3/2010
  Version          : 1.0
  Copyright        : Soft'n'design
  Author           : Laurent Mazet <mazet@softndesign.org>

  Description      : This file contains verbose macros

  History          :
  - initial version
*/

#ifndef __VERBOSE_H__
#define __VERBOSE_H__

/**
   Declare a verbose level. Should be used only one time per module
   outside the main function.

   @param module module name
   @param init initial value
*/
#define DECLARE_VERBOSE_LEVEL(module, init) \
    int verbose_##module = init;

/**
   Change a verbose level.

   @param module module name
   @param level verbose level
*/
#define CHANGE_VERBOSE_LEVEL(module, level) \
    do {                                    \
        extern int verbose_##module;        \
        verbose_##module = level;           \
    } while(0)
        
/**
   Execute a statement (mostly a message display) depending of a
   level. Level is defined by global variable verbose_`module`.

   @param module module name
   @param level verbose level
   @param statement controled statement
*/
#define VERBOSE(module, level, statement...) \
    do {                                     \
        extern int verbose_##module;         \
        if (level <= verbose_##module) {     \
            statement;                       \
        }                                    \
    } while(0)

#endif /* __VERBOSE_H__ */

/* vi:set tabstop=4 expandtab shiftwidth=4: this line set vi mode */
