/*
  File name        : task.h
  Date of creation : 25/01/2010
  Version          : 1.0
  Copyright        : Soft'n'Design
  Author           : Laurent Mazet <mazet@softndesign,org>

  Description      : Task management

  History          :
  - initial version
*/

#ifndef __TASK_H__
#define __TASK_H__

#include <pthread.h>

__BEGIN_DECLS

struct _task_t;

typedef struct {
    pthread_t thread;
    pthread_mutex_t condition_mutex;
    pthread_cond_t condition_cond;
    int trigger;
    struct _task_t *task;
    int sub_task_id;

    int count;

    unsigned long *start_time_us;
    unsigned long *exec_time_us;
} sub_task_t;


typedef struct _task_t {
    char *name;
    int is_sync;
    int priority;
    char *cpu_list;

    int (*task) (sub_task_t *s, int subtask_id);
    long long max_allowed_time;

    sub_task_t *sub_task_list;
    int nb_sub_tasks;

    // Init function

    int (*init) (sub_task_t *s, int subtask_id);
    pthread_mutex_t init_mutex;

    // Statistics section

    pthread_mutex_t stats_mutex;

    int count;
    long long min_exec_time;
    long long max_exec_time;
    long long current_exec_time;
    unsigned long long cumul_exec_time;
    unsigned long int overtimes;
    unsigned long int overloads;
} task_t;

task_t *create_sync_task (const char *name, int (*task) (sub_task_t *, int),
                          float max_allowed_time_us, int (*init) (sub_task_t *, int),
                          int priority, int nb_sub_tasks, const char *cpu_list);

task_t *create_async_task (const char *name, int (*task) (sub_task_t *, int),
                           int priority, int nb_sub_tasks, const char *cpu_list);

int task_trig (task_t * t);

int task_try_trig (task_t * t);

int task_end (task_t * t);

int task_selective_run (task_t * t, int run_list[]);

void task_stat (task_t * t, int condensed);

#endif /* __TASK_H__ */

/* vi:set tabstop=4 expandtab shiftwidth=4: this line set vi mode */
